@@ -6,6 +6,7 @@ import logging
 
 logger = logging.getLogger(__name__)
 
+
 class AutoViewFieldMixin(object):
     """
     Registers itself with AutoResponseView.
@@ -41,7 +42,12 @@ class AutoViewFieldMixin(object):
             logger.info("Registering auto field: %s", name)
 
         from . import util
-        id_ = util.register_field(name, self)
+
+        rf = util.register_field
+        if logger.isEnabledFor(logging.DEBUG):
+            rf = util.timer(rf)
+
+        id_ = rf(name, self)
         self.field_id = id_
         super(AutoViewFieldMixin, self).__init__(*args, **kwargs)
 
@@ -77,14 +83,14 @@ from django.forms.models import ModelChoiceIterator
 from django.db.models import Q
 from django.utils.translation import ugettext_lazy as _
 from django.utils.encoding import smart_unicode
-from django.core.validators import EMPTY_VALUES
 
 from .widgets import Select2Widget, Select2MultipleWidget,\
     HeavySelect2Widget, HeavySelect2MultipleWidget, AutoHeavySelect2Widget, \
-    AutoHeavySelect2MultipleWidget
+    AutoHeavySelect2MultipleWidget, AutoHeavySelect2Mixin
 from .views import NO_ERR_RESP
 from .util import extract_some_key_val
 
+
 ### Light general fields ###
 
 class Select2ChoiceField(forms.ChoiceField):
@@ -112,6 +118,9 @@ class ModelResultJsonMixin(object):
 
     It is expected that sub-classes will defined a class field variable
     ``search_fields``, which should be a list of field names to search for.
+
+    ..note:: As of version 3.1.3, ``search_fields`` is optional if sub-class
+        overrides ``get_results``.
     """
 
     def __init__(self, *args, **kwargs):
@@ -129,12 +138,9 @@ class ModelResultJsonMixin(object):
             value. (Default is ``pk``, i.e. the id field of the model)
         :type to_field_name: :py:obj:`str`
         """
-        if self.queryset is None and not kwargs.has_key('queryset'):
+        if self.queryset is None and not 'queryset' in kwargs:
             raise ValueError('queryset is required.')
 
-        if not self.search_fields:
-            raise ValueError('search_fields is required.')
-
         self.max_results = getattr(self, 'max_results', None)
         self.to_field_name = getattr(self, 'to_field_name', 'pk')
 
@@ -196,7 +202,7 @@ class ModelResultJsonMixin(object):
 
             | Assume, ``search_term == 'John'``
             | ``self.search_fields == ['first_name__icontains', 'last_name__icontains']``
-            
+
             So, the prepared query would be::
 
                 {
@@ -206,7 +212,7 @@ class ModelResultJsonMixin(object):
                     'and': {}
                 }
         :rtype: :py:obj:`dict`
-        """      
+        """
         q = None
         for field in search_fields:
             kwargs = {}
@@ -215,7 +221,7 @@ class ModelResultJsonMixin(object):
                 q = Q(**kwargs)
             else:
                 q = q | Q(**kwargs)
-        return {'or': [q], 'and': {},}
+        return {'or': [q], 'and': {}}
 
     def get_results(self, request, term, page, context):
         """
@@ -223,12 +229,15 @@ class ModelResultJsonMixin(object):
 
         This implementation takes care of detecting if more results are available.
         """
+        if not hasattr(self, 'search_fields') or not self.search_fields:
+            raise ValueError('search_fields is required.')
+
         qs = copy.deepcopy(self.queryset)
         params = self.prepare_qs_params(request, term, self.search_fields)
 
         if self.max_results:
             min_ = (page - 1) * self.max_results
-            max_ = min_ + self.max_results + 1 # fetching one extra row to check if it has more rows.
+            max_ = min_ + self.max_results + 1  # fetching one extra row to check if it has more rows.
             res = list(qs.filter(*params['or'], **params['and'])[min_:max_])
             has_more = len(res) == (max_ - min_)
             if has_more:
@@ -237,7 +246,7 @@ class ModelResultJsonMixin(object):
             res = list(qs.filter(*params['or'], **params['and']))
             has_more = False
 
-        res = [ (getattr(obj, self.to_field_name), self.label_from_instance(obj), ) for obj in res ]
+        res = [(getattr(obj, self.to_field_name), self.label_from_instance(obj), ) for obj in res]
         return (NO_ERR_RESP, has_more, res, )
 
 
@@ -258,7 +267,7 @@ class UnhideableQuerysetType(type):
             # class variable named - queryset, which will
             # effectively hide the queryset declared in this
             # mixin.
-            dct.pop('queryset') # Throwing away the sub-class queryset
+            dct.pop('queryset')  # Throwing away the sub-class queryset
             dct['_subclass_queryset'] = _q
 
         return type.__new__(cls, name, bases, dct)
@@ -290,7 +299,8 @@ class ChoiceMixin(object):
 
     def __deepcopy__(self, memo):
         result = super(ChoiceMixin, self).__deepcopy__(memo)
-        result._choices = copy.deepcopy(self._choices, memo)
+        if hasattr(self, '_choices'):
+            result._choices = copy.deepcopy(self._choices, memo)
         return result
 
 
@@ -326,10 +336,11 @@ class ModelChoiceFieldMixin(object):
             'empty_label', 'cache_choices', 'required', 'label', 'initial', 'help_text',
             ])
         kargs['widget'] = kwargs.pop('widget', getattr(self, 'widget', None))
-        kargs['to_field_name'] = kwargs.pop('to_field_name', 'pk')        
+        kargs['to_field_name'] = kwargs.pop('to_field_name', 'pk')
 
-        if hasattr(self, '_choices'): # If it exists then probably it is set by HeavySelect2FieldBase.
-                                      # We are not gonna use that anyway.
+        # If it exists then probably it is set by HeavySelect2FieldBase.
+        # We are not gonna use that anyway.
+        if hasattr(self, '_choices'):
             del self._choices
 
         super(ModelChoiceFieldMixin, self).__init__(queryset, **kargs)
@@ -352,9 +363,10 @@ class ModelMultipleChoiceField(ModelChoiceFieldMixin, forms.ModelMultipleChoiceF
     queryset = property(ModelChoiceFieldMixin._get_queryset, forms.ModelMultipleChoiceField._set_queryset)
 
 
-### Light Fileds specialized for Models ###
+### Light Fields specialized for Models ###
 
-class ModelSelect2Field(ModelChoiceField) :
+
+class ModelSelect2Field(ModelChoiceField):
     """
     Light Select2 field, specialized for Models.
 
@@ -363,7 +375,7 @@ class ModelSelect2Field(ModelChoiceField) :
     widget = Select2Widget
 
 
-class ModelSelect2MultipleField(ModelMultipleChoiceField) :
+class ModelSelect2MultipleField(ModelMultipleChoiceField):
     """
     Light multiple-value Select2 field, specialized for Models.
 
@@ -399,14 +411,16 @@ class HeavySelect2FieldBaseMixin(object):
             be raised.
 
         """
+        from . import util
+        if logger.isEnabledFor(logging.DEBUG):
+            t1 = util.timer_start('HeavySelect2FieldBaseMixin.__init__')
+
         data_view = kwargs.pop('data_view', None)
         choices = kwargs.pop('choices', [])
 
         kargs = {}
-        if data_view is not None:
+        if kwargs.get('widget', None) is None:
             kargs['widget'] = self.widget(data_view=data_view)
-        elif kwargs.get('widget', None) is None:
-            raise ValueError('data_view is required else you need to provide your own widget instance.')
 
         kargs.update(kwargs)
         super(HeavySelect2FieldBaseMixin, self).__init__(*args, **kargs)
@@ -415,16 +429,22 @@ class HeavySelect2FieldBaseMixin(object):
 
         # This piece of code is needed here since (God knows why) Django's Field class does not call
         # super(); because of that __init__() of classes would get called after Field.__init__().
-        # If did had super() call there then we could have simply moved AutoViewFieldMixin at the
+        # If it did had super() call there then we could have simply moved AutoViewFieldMixin at the
         # end of the MRO list. This way it would have got widget instance instead of class and it
         # could have directly set field_id on it.
         if hasattr(self, 'field_id'):
             self.widget.field_id = self.field_id
 
-        if not choices and hasattr(self, 'choices'): # ModelChoiceField will set this to ModelChoiceIterator
-            choices = self.choices
-        self.choices = choices
+        if logger.isEnabledFor(logging.DEBUG):
+            t2 = util.timer_start('HeavySelect2FieldBaseMixin.__init__:choices initialization')
 
+        # ModelChoiceField will set self.choices to ModelChoiceIterator
+        if choices and not (hasattr(self, 'choices') and isinstance(self.choices, forms.models.ModelChoiceIterator)):
+            self.choices = choices
+
+        if logger.isEnabledFor(logging.DEBUG):
+            util.timer_end(t2)
+            util.timer_end(t1)
 
 class HeavyChoiceField(ChoiceMixin, forms.Field):
     """
@@ -489,7 +509,7 @@ class HeavyChoiceField(ChoiceMixin, forms.Field):
         try:
             value = self.coerce_value(value)
             self.validate_value(value)
-        except Exception, e:
+        except Exception:
             logger.exception("Exception while trying to get label for value")
             return None
         return self.get_val_txt(value)
@@ -507,6 +527,7 @@ class HeavyChoiceField(ChoiceMixin, forms.Field):
         """
         return None
 
+
 class HeavyMultipleChoiceField(HeavyChoiceField):
     """
     Reimplements :py:class:`django.forms.TypedMultipleChoiceField` in a way which suites the use of big data.
@@ -582,11 +603,6 @@ class AutoSelect2Field(AutoViewFieldMixin, HeavySelect2ChoiceField):
 
     widget = AutoHeavySelect2Widget
 
-    def __init__(self, *args, **kwargs):
-        self.data_view = "django_select2_central_json"
-        kwargs['data_view'] = self.data_view
-        super(AutoSelect2Field, self).__init__(*args, **kwargs)
-
 
 class AutoSelect2MultipleField(AutoViewFieldMixin, HeavySelect2MultipleChoiceField):
     """
@@ -600,11 +616,6 @@ class AutoSelect2MultipleField(AutoViewFieldMixin, HeavySelect2MultipleChoiceFie
 
     widget = AutoHeavySelect2MultipleWidget
 
-    def __init__(self, *args, **kwargs):
-        self.data_view = "django_select2_central_json"
-        kwargs['data_view'] = self.data_view
-        super(AutoSelect2MultipleField, self).__init__(*args, **kwargs)
-
 
 ### Heavy field, specialized for Model, that uses central AutoView ###
 
@@ -615,16 +626,12 @@ class AutoModelSelect2Field(ModelResultJsonMixin, AutoViewFieldMixin, HeavyModel
     This needs to be subclassed. The first instance of a class (sub-class) is used to serve all incoming
     json query requests for that type (class).
     """
-    __metaclass__ = UnhideableQuerysetType # Makes sure that user defined queryset class variable is replaced by
-                                           # queryset property (as it is needed by super classes).
+    # ModelChoiceField will set this to ModelChoiceIterator
+    # queryset property (as it is needed by super classes).
+    __metaclass__ = UnhideableQuerysetType
 
     widget = AutoHeavySelect2Widget
 
-    def __init__(self, *args, **kwargs):
-        self.data_view = "django_select2_central_json"
-        kwargs['data_view'] = self.data_view
-        super(AutoModelSelect2Field, self).__init__(*args, **kwargs)
-
 
 class AutoModelSelect2MultipleField(ModelResultJsonMixin, AutoViewFieldMixin, HeavyModelSelect2MultipleChoiceField):
     """
@@ -633,14 +640,9 @@ class AutoModelSelect2MultipleField(ModelResultJsonMixin, AutoViewFieldMixin, He
     This needs to be subclassed. The first instance of a class (sub-class) is used to serve all incoming
     json query requests for that type (class).
     """
-    __metaclass__ = UnhideableQuerysetType # Makes sure that user defined queryset class variable is replaced by
-                                           # queryset property (as it is needed by super classes).
+    # Makes sure that user defined queryset class variable is replaced by
+    # queryset property (as it is needed by super classes).
+    __metaclass__ = UnhideableQuerysetType
 
     widget = AutoHeavySelect2MultipleWidget
 
-    def __init__(self, *args, **kwargs):
-        self.data_view = "django_select2_central_json"
-        kwargs['data_view'] = self.data_view
-        super(AutoModelSelect2MultipleField, self).__init__(*args, **kwargs)
-
-
